#!/usr/bin/env python3

import re
import sys
import time
import json

# creates ncdu-format indexes from various input formats,
#   `lftp` => `find -l`
#   `rsync -av --list-only`
#
# examples:
#   tr '\r' '\n' <lftp-find-l.txt | ./ls2ncdu.py | ncdu -f-
#   rsync -av --list-only rsync://mirrors.kernel.org/mirrors/alpine/ | ./ls2ncdu.py | ncdu -f-

def main():
	# lftp:  -rw-r--r--     2507254 1997-01-17 10:54:30 ./Inet/N16E20.EXE
	# rsync: -rw-r--r--  46,137,344 2021/08/27 11:11:28 v3.14/releases/x86_64/alpine-virt-3.14.2-x86_64.iso
	ptn = re.compile(r'^[^d]r[w-][x-]r[w-][x-]r[w-][x-] +([0-9,]+) [0-9/-]{10} [0-9:]{8} (?:\./)?(.*)')
	print(f'[1,1,{{"progname":"ls2ncdu.py","progver":"0.1","timestamp":{int(time.time())}}},')
	print('[{"name":".","asize":3,"dsize":512,"dev":1,"ino":11}', end='')
	stk = []
	sep = ''
	ino = 11
	for ln in sys.stdin:
		m = ptn.match(ln.rstrip())
		if not m:
			continue

		sz, fp = m.groups()
		if " -> " in fp:
			continue

		sz = int(sz.replace(',', ''))
		fp = fp.split('/')
		
		sep = ',\n'
		comm = 0
		for d1, d2 in zip(stk, fp[:-1]):
			if d1 != d2:
				break
			comm += 1
		rm = len(stk) - comm
		if rm:
			print()
		for _ in range(rm):
			stk.pop()
			print(']', end="")
		for d in fp[comm:-1]:
			ino += 1
			stk.append(d)
			print(f',\n[{{"name":{json.dumps(d)},"asize":3,"dsize":512,"ino":{ino}}}', end='')
		
		ino += 1
		print(f',\n{{"name":{json.dumps(fp[-1])},"asize":{sz},"dsize":{sz},"ino":{ino}}}', end='')

	for d in stk:
		print(']', end="")
	print('\n]]')


if __name__ == '__main__':
	main()
